import { Component, h } from "preact";
import * as EasyMDE from "easymde";
// import "./easymde.css";
// import "./dark.css";
import "easymde/dist/easymde.min.css";
// import "codemirror/theme/monokai.css";

export interface EditorProps {
    value: string;
    valueChange: (value: string) => void;
    save: () => void;
    exit: () => void;
}

export default class Editor extends Component<EditorProps, {}> {
    element: HTMLInputElement;
    easymde: EasyMDE;

    componentDidMount() {
        if (this.easymde)
            this.easymde.toTextArea();

        this.easymde = new EasyMDE({
            element: this.element,
            initialValue: this.props.value,
            autofocus: true,
            spellChecker: false
            // theme: "monokai"
            // autoDownloadFontAwesome: true
        })

        this.easymde.codemirror.on("change", () => {
            this.props.valueChange(this.easymde.value());
        });

        this.easymde.codemirror.addKeyMap({
            "Ctrl-S": () => this.props.save(),
            "Esc": () => this.props.exit()
        })
    }

    shouldComponentUpdate(props: EditorProps) {
        if (this.easymde.value() !== props.value)
            this.easymde.value(props.value);
        return false;
    }

    render() {
        return <textarea ref={ta => this.element = ta}></textarea>
    }
}