import { h, Component } from "preact"
import { IVault, ViewNote } from "../../../notes";
import Trash from "feather-icons/dist/icons/trash-2.svg"
import X from "feather-icons/dist/icons/x.svg"
import Save from "feather-icons/dist/icons/save.svg"

import Navigation from "../../../navigation";
import { YesNoModal } from "../../modals/YesNoModal";
import Notifications, { MessageType } from "../../../notifications";
import Theme from "../../../theme";

import Editor from "./Editor";

const minRows = 3;
export default class EntryComponent extends Component<{ vault: Promise<IVault>, id: string | undefined, note: string | undefined }, { title: string, changed: boolean }> {
   private text: string = "";
   private vault: IVault;
   // private lineHeight: number = 24;
   private note: ViewNote;

   private skip_save: boolean = false;


   constructor(props) {
      super(props);
      this.state = { changed: false, title: "" };
   }

   private toVault() {
      history.back();
   }

   componentDidMount() { }

   async componentWillMount() {
      this.text = "";
      this.vault = undefined;
      this.note = undefined;
      this.skip_save = false;
      this.setState({ changed: false, title: "" });
      try {
         this.vault = await this.props.vault;
         let note: ViewNote;
         let changed = false;
         if (this.props.id)
            note = await this.vault.getNote(this.props.id)
         else {
            note = this.vault.newNote();
            if (this.props.note) {
               note.__value = this.props.note;
               changed = true;
            }
         }

         if (!note) {
            Notifications.sendNotification("Note not found!", MessageType.ERROR);
         } else {
            this.note = note;
            this.text = note.__value;

            let [title] = this.text.split("\n", 1);
            this.setState({ title, changed })
         }
      } catch (err) {
         Notifications.sendError(err);
      }
   }

   private async save() {
      try {
         if (this.state.changed) {
            this.note.__value = this.text;
            await this.vault.saveNote(this.note);
            this.setState({ changed: false })
         }
      } catch (err) {
         Notifications.sendError(err);
      }
   }

   componentWillUnmount() {
      if (!this.skip_save)
         this.save()
   }

   private textChange(value: string) {
      if (!this.state.changed && value !== this.text)
         this.setState({ changed: true })

      this.text = value;

      let [title] = value.split("\n", 1);
      if (title !== this.state.title)
         this.setState({ title });
   }

   private async exitHandler() {
      if (this.state.changed) {
         let modal = new YesNoModal("Really want to quit?");
         let res = await modal.getResult();
         if (res === true) {
            this.skip_save = true;
            this.toVault();
         }
      } else
         this.toVault()
   }

   render() {
      const save_handler = async () => {
         await this.save()
         Navigation.setPage("/vault", { id: this.vault.id }, { entry: "false" }, true);
         this.toVault()
      }

      const delete_handler = async () => {
         await this.vault.deleteNote(this.props.id);
         this.toVault()
      }

      console.log("Rerender")

      return <div>
         <header class="header">
            <a class="header-icon-button" onClick={() => this.exitHandler()}><X height={undefined} width={undefined} /></a>
            {this.state.changed ? <a class="header-icon-button" style="margin-left: 0.5em;" onClick={() => save_handler()}><Save height={undefined} width={undefined} /></a> : undefined}
            <h3 style="display:inline" class="button header-title">{this.state.title}</h3>
            <a class="header-icon-button" onClick={() => delete_handler()}><Trash height={undefined} width={undefined} /></a>
         </header>
         <div class="container" style={Theme.isDark() ? "filter: invert(100%);" : ""}>
            <Editor
               value={this.text}
               valueChange={value => this.textChange(value)}
               save={() => this.save()}
               exit={() => this.exitHandler()} />
         </div>
      </div>
   }
}